//
// ringbufferpuller.h
// Copyright Phase Five Systems
//

#include "jump/base/logger.h"
#include "jump/base/sharedmem.h"
#include "jump/protocol/base/rdclientbase.h"
#include "jump/protocol/rtc/audiocapture.h"

#pragma once

namespace jump
{

//
// AudioRingBufferPuller pulls audio from an audio stream puller and
// writes the pulled audio into the a LocalAudioCaptureMappedRingBuffer
//
class AudioRingBufferPublisher
{
public:
    AudioRingBufferPublisher(Logger::LogCompRef parentLogger);
    virtual ~AudioRingBufferPublisher();

    void SetPuller(std::shared_ptr<RDClientAudioStreamPuller> puller);
    void SetRingBuffer(
        std::shared_ptr<LocalAudioCaptureMappedRingBuffer> buffer,
        std::shared_ptr<SharedMem> optionalBufferBackingSharedMem);
    void Start();
    void Stop();

    void SetDebugOutputFilePath(const std::string &outputFilePath);

protected:
    void PullAudio();

    Logger::LogCompRef mLogger;
    std::shared_ptr<RDClientAudioStreamPuller> mPuller;
    std::shared_ptr<LocalAudioCaptureMappedRingBuffer> mRingBuffer;
    std::shared_ptr<SharedMem> mRingBufferSharedMem;
    std::unique_ptr<ThreadRunnerStdThread> mPullerThread;
    TimeVal mPollInterval;
    RunnerPoller mAudioPullPoller;
    std::string mDebugOutputFilePath;
    RDClientAudioStreamDescription mAudioDesc;
    CancelableRunner mRunner;
    SignalConnections mConns;
};

//
// AudioRingBufferServer is used to serve frames that are captured
// by AudioRingBufferPuller into a LocalAudioCaptureMappedRingBuffer
class AudioRingBufferConsumer
{
public:
    AudioRingBufferConsumer(Logger::LogCompRef parentLogger);

    void OnGetAudio(const RDClientAudioStreamDescription &bufferDesc,
                    int frames,
                    void *destBuffer,
                    LocalAudioCaptureMappedRingBuffer &buffer);

protected:
    void ReduceLagIfNeeded(const RDClientAudioStreamDescription &bufferDesc,
                           LocalAudioCaptureMappedRingBuffer &buffer,
                           const TimeVal &audioProducerPushInterval,
                           const TimeVal &audioReaderPullInterval);

    Logger::LogCompRef mLogger;
};
} // namespace jump
